import java.util.LinkedList;

/**
 * Classe para representar uma video disk
 * 
 * @author Vasco T. Vasconcelos
 * @version 1.1 19/Jul/2013 as 
 */
public class GeneralVideoDisk implements Cloneable {

	/**
	 * Programas gravados
	 */
    private LinkedList<Program> programs;

    /**
     * Capacidade
     */
    private int length;

    /**
     * Construtor 
     * @param length - capacidade deste GeneralVideoDisk
     */
    public GeneralVideoDisk(int length) {
        this.programs = new LinkedList<Program>();
        this.length = length;
    }

    /**
     * Grava o programa indicado no GeneralVideoDisk
     * 
     * @param p - programa a gravar
     */
    public void record(Program p) {
        programs.add(p);
    }

    /**
     * Apaga o programa indicado do GeneralVideoDisk
     * 
     * @param p - programa a apagar
     */
    public void erase(Program p) {
        programs.remove(p);
    }

    /**
     * A duracao do programa indicado
     * 
     * @param p - programa pretendido
     * @return a duracao do programa 
     */
    public int duration(Program p) {
        return p.getLength();
    }

    /**
     * O titulo do programa indicado
     * 
     * @param p - programa pretendido
     * @return o titulo do programa 
     */
    public String title(Program p) {
        return p.getTitle();
    }

    /**
     * O indice do ultimo programa gravado neste GeneralVideoDisk
     * 
     * @return indice do ultimo programa
     */
    public int lastIndex() {
        return programs.size() - 1;
    }

    /**
     * Devolve o tempo que resta neste GeneralVideoDisk
     * 
     * @return tempo restante 
     */
    public int remainingTime() {
        int used = 0;
        int size = programs.size();
        for (int i = 0; i < size; i++) {
            Program prog = programs.get(i);
            used += prog.getLength();
        }
        return length - used;
    }

    /**
     * Devolve o indice do programa indicado
     * 
     * @param prog - o programa procurado
     * @return o indice do programa caso exista, -1 caso contrario
     */
    public int indexOf(Program prog) {
        int result = 0;
        int size = programs.size();
        for (int i = 0; i < size; i++) {
            Program other = programs.get(i);
            if (other.getTitle().equals(prog.getTitle()))
                return result;
            result++;
        }
        return -1;
    }

    /**
     * Verifica se este GeneralVideoDisk esta vazio
     * 
     * @return true se estiver vazio, false caso contrario
     */
    public boolean isEmpty() {
        return programs.size() == 0;
    }

    /**
     * A capacidade deste GeneralVideoDisk
     * 
     * @return capacidade deste GeneralVideoDisk
     */
    public int capacity() {
        return length;
    }

    /**
     * Devolve o programa gravado com o indice indicado
     * 
     * @param index - o indice do programa
     * @return - o Program pretendido se existir
     */
    public Program recording(int index) {
        return programs.get(index);
    }

    /**
     * Devolve o programa gravado com o nome indicado
     * 
     * @param title - o titulo do programa
     * @return - o Program pretendido se existir, null caso contrario
     */
    public Program recording(String title) {
        for (Program p : programs)
            if (p.getTitle().equals(title))
                return p;
        return null;
    }

    /**
     * Devolve uma copia deste GeneralVideoDisk
     * 
     * @return clone deste GeneralVideoDisk
     */
    @SuppressWarnings("unchecked")
	public Object clone() {
        try {
            GeneralVideoDisk result = (GeneralVideoDisk) super.clone();
            result.programs = (LinkedList<Program>) programs.clone();
            return result;
        } catch (CloneNotSupportedException e) {
            throw new InternalError(e.getMessage());
        }
    }

    /**
     * Devolve o hash code para este Program
     * 
     * @return o hash code para este programa
     */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + length;
		result = prime * result
				+ ((programs == null) ? 0 : programs.hashCode());
		return result;
	}

    /**
     * Este objeto eh igual ao dado?
     * 
     * @param other - o objeto a comparar
     * @return true se os objetos forem iguais, false caso contrario
     */
	@Override
    public boolean equals(Object other) {
        return other == this || other != null && this.getClass() == other.getClass()
                && length == ((GeneralVideoDisk) other).length
                && programs.equals(((GeneralVideoDisk) other).programs);
    }
	
	 /**
     * A representacao textual de GeneralVideoDisk
     * 
     * @return String que representa este GeneralVideoDisk
     */
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("GeneralVideoDisk:\n");
		sb.append('[');
		for (Program prog : programs) {
			sb.append(prog + ";");
		}
		sb.deleteCharAt(sb.length()-1);
		sb.append(']');
		return sb.toString();
	}   
}
